/*Siddhartha Kasivajhula
* Georgia Institute of Technology
* c/o Predrag Cvitanovic
* Advisor: Rytis Paskauskas
* Fall 2005
*/


public class Surface {
    public static final double L = 1;                                            // Length of surface ("period")
    static final double INTERVAL = L / (AFMConstants.SAMPLES - 1);                // divide length into 1000 samples
    private double[] x;                                                                // x
    private double[] y;                                                                // y
    private double[] dy;  //here you are, thinking about derivatives	// slope
    private double A;                                                                    // Amplitude
    private double p;                                                                    // Phase
    private double tx[];                                                                // Normalised tangent vector: x component
    private double ty[];                                                                // Normalised tangent vector: y component
    private double nx[];                                                                // Normalised normal vector: x component
    private double ny[];                                                                // Normalised normal vector: y component

    public Surface()                                            // Constructor
    {            // sets up arrays, defines surface function
        x = new double[AFMConstants.SAMPLES];
        y = new double[AFMConstants.SAMPLES];
        dy = new double[AFMConstants.SAMPLES];
        tx = new double[AFMConstants.SAMPLES];
        ty = new double[AFMConstants.SAMPLES];
        nx = new double[AFMConstants.SAMPLES];
        ny = new double[AFMConstants.SAMPLES];
        A = 0.038533472746501492;        // amplitude
        p = -0.67113170769742891 + 0.5 - 0.25025025025025052494942851164536;//-0.17113170769742891;//0.6385710978101264;//-0.17113170769742891;//0.3288682923025711;//0.13857109781012633;//0.67113170769742891//+0.3288682923025711;//+0.17401688954879357;//-2*0.016280304943651295;		// phase
        for (int i = 0; i < AFMConstants.SAMPLES; i++)        // initialises all values: x,y,slope,tangent and normal vectors
        {
            x[i] = INTERVAL * i;
            y[i] = A * Math.sin(2 * Math.PI / Surface.L * (x[i] - p * Surface.L));
            dy[i] = A * 2 * Math.PI / Surface.L * Math.cos(2 * Math.PI / Surface.L * (x[i] - p * Surface.L));
            tx[i] = 1 / Math.pow((dy[i] * dy[i] + 1), 0.5);
            ty[i] = dy[i] * tx[i];                                        // ty = m*tx
            nx[i] = -ty[i];                                            // for normal, simply switch coefficients and negate x coefficient
            ny[i] = tx[i];                                                //
        }
    }//end constructor

    public String getType() {
        return "sin";
    }
    public double getA() {
        return A;
    }

    public double getP() {
        return p;
    }

    public double getL() {
        return L;
    }

    public double gimmeY(double x)    // returns precise y for a precise x
    {
        return (A * Math.sin(2 * Math.PI / Surface.L * (x - p * Surface.L)));
    }

    public double getX(int index) {
        return x[index];
    }

    public double getY(int index) {
        return y[index];
    }

    public double getY(double x)        // return y given x. Not precise:
    {                                        // returns y value at closest index
        int temp = (int) Math.round(x / L * AFMConstants.SAMPLES);
        do {
            if (temp >= AFMConstants.SAMPLES) temp = temp - AFMConstants.SAMPLES; // in case the AFM is beyond
            if (temp < 0) temp = temp + AFMConstants.SAMPLES;    // surface array boundaries, y value can still
        } while (!(temp >= 0 && temp < AFMConstants.SAMPLES));
        return y[temp];                                            // be found by "wrapping around"
    }

    public double getSlope(int index) {
        return dy[index];
    }

    public double getNX(int index)    // x- component of normal vector *at a particular array index*
    {
        return nx[index];
    }

    public double getNY(int index)    // y-component of normal vector *at a particular array index*
    {
        return ny[index];
    }

    public double getNX(double x)        // return precise x-component of normal vector for *any* given precise x
    {
        double yTemp = A * Math.sin(2 * Math.PI / Surface.L * (x - p * Surface.L));    // equation of the surface
        double dyTemp = A * 2 * Math.PI / Surface.L * Math.cos(2 * Math.PI / Surface.L * (x - p * Surface.L));    // derivative
        double txTemp = 1 / Math.pow((dyTemp * dyTemp + 1), 0.5);    // tangent x
        double tyTemp = dyTemp * txTemp;                                        // ty = m*tx
        double nxTemp = -tyTemp;                                            // for normal, simply switch coefficients and negate x coefficient
        return nxTemp;
    }

    public double getNY(double x)        // return precise y-component of normal vector for *any* given precise x
    {
        double yTemp = A * Math.sin(2 * Math.PI / Surface.L * (x - p * Surface.L));
        double dyTemp = A * 2 * Math.PI / Surface.L * Math.cos(2 * Math.PI / Surface.L * (x - p * Surface.L));
        double txTemp = 1 / Math.pow((dyTemp * dyTemp + 1), 0.5);
        double tyTemp = dyTemp * txTemp;                                        // ty = m*tx
        double nyTemp = txTemp;                                                // for normal, simply switch coefficients and negate x coefficient
        return nyTemp;
    }

    public static void main(String[] args)        // for testing
    {
        Surface sine = new Surface();
        System.out.println(sine.gimmeY(-0.01628030494365129));
    }
}// end Surface
